import re
from .base_label import BaseLabel

class ShopifyLabel(BaseLabel):
    def __init__(self, page_text, page_table,page_num):
        super().__init__(page_text, page_table,page_num)
        self.shopify_order_id_pattern = r'#\d{4,5}'
        self.product_name_pattern = r'ITEMS QUANTITY\n(.*)\nThank you for shopping with us'
        self.qty_pattern = r'(\d+)\sof\s\d+'
        
        """
        # Common 
        self.page_debrief_dict = {
            "order_id" : None, "sorting_key" : None, "qty" : None
        }
        """
    
    def analyze_shpy_page(self):
        try:
            #print(page_text)
            id_match = re.findall(self.shopify_order_id_pattern, self.page_text)
            
            prod_desc_match = re.findall(self.product_name_pattern, self.page_text,flags=re.DOTALL)
            if id_match:
                self.page_debrief_dict["order_id"] = id_match[0]
            
            if prod_desc_match:
                # find the product name in string form
                prod_desc_str = prod_desc_match[0].replace("\n"," ")
                
                # check it the order is mixed
                item_count = len(re.findall(self.qty_pattern,prod_desc_str))
                    
                if item_count == 1:
                    self.page_debrief_dict["qty"] = re.search(self.qty_pattern, prod_desc_str).group(1)
                    self.page_debrief_dict["sorting_key"] = re.sub(self.qty_pattern,'',prod_desc_str)
                else:
                    self.page_debrief_dict["sorting_key"] = "Mixed"
        except Exception as e:
            print(e)
        else:
            return self.page_debrief_dict
    




class BaseLabel:
    def __init__(self, page_text, page_table, page_num = None):
        self.page_debrief_dict = {
            "order_id" : None, "sorting_key" : None, "qty" : None
        }
        self.page_text = page_text
        self.page_table = page_table
        self.page_number = page_num
    


import pdfplumber, re, os,sys, logging, json
from pypdf import PdfReader, PdfWriter
from pprint import pprint
from label_sorter.platforms.ecommerce.base_label import BaseLabel
from label_sorter.platforms.ecommerce.shopify import ShopifyLabel
from label_sorter.platforms.ecommerce.amazon import AmazonLabel

logging.getLogger('pdfminer').setLevel(logging.ERROR)

class LabelSorter:
    def __init__(self, pdf_path):
        self.sorted_dict = {}
        self.label_filepath = pdf_path
        self.output_folder = self.label_filepath.replace(".pdf","")
        self.platform = self.find_platform()
        
    def find_platform(self) -> str:
        platform = None
        try:
            with pdfplumber.open(self.label_filepath) as pdf_file:
                total_pages = 0; amazon_count = 0 
                
                shopify_order_id_count, amazon_order_id_count = 0, 0
                
                for page_index, page in enumerate(pdf_file.pages):
                    total_pages += 1
                    page_text = page.extract_text(); page_tables = page.extract_tables()
                    
                    # Shopify Initializations
                    sh = ShopifyLabel(page_text=page_text, page_table=page_tables,page_num=0)
                    am = AmazonLabel(page_text=page_text, page_table=page_tables,page_num=0)
                    
                    if re.findall(sh.shopify_order_id_pattern, page_text):
                        shopify_order_id_count += 1
                    elif re.findall(am.amazon_order_id_pattern, page_text):
                        amazon_order_id_count += 1
                        
                if total_pages == shopify_order_id_count:
                    platform = "Shopify"
                # this condition is not complete, need to add overlap page detection
                elif amazon_order_id_count > 0:
                    platform = "Amazon"
            
        except FileNotFoundError:
            print(f"The file {self.label_filepath} does not exist.")
        except Exception as e:
            print(e)
        else:
            return platform
        
    def create_sorted_summary(self):
        if not self.platform:
            sys.exit("Unsupported Platform, exiting....")
        page_debrief = None
        try:
            print(f"Platform : {self.platform}")
            with pdfplumber.open(self.label_filepath) as pdf_file:
                for page_index, page in enumerate(pdf_file.pages):
                    page_text = page.extract_text(); page_table = page.extract_tables()
                    page_number = page_index+1
                    
                    #Label_instance = BaseLabel(page_text=page_text, page_table=page_table,page_num=page_number)
                    debriefs = {
                        "Shopify" : ShopifyLabel(page_text=page_text, page_table=page_table,page_num=page_number).analyze_shpy_page(),
                        "Amazon" : AmazonLabel(page_text=page_text, page_table=page_table,page_num=page_number).analyze_amzn_page(),
                    }
                    
                    page_debrief = debriefs[self.platform]
                    
                    print(f"{page_number} : {page_debrief}")
                    
                    is_page_debrief_populated = page_debrief["order_id"] != None
                    # sorting summary
                    if self.platform and is_page_debrief_populated:
                        self.populate_shipment_summary(
                            sorting_key=page_debrief["sorting_key"], qty=page_debrief["qty"],
                            page_nums=[page_number - 1, page_number] if self.platform == "Amazon" else [page_number]
                        )
                    
        except FileNotFoundError as fe:
            print(fe)
        except Exception as e:
            print(e)
        else:
            return self.sorted_dict
        
    def populate_shipment_summary(self, sorting_key:str, page_nums:list, qty : str) -> None:
        try:
            # different conditions for mixed and single items
            # sorting key initialization
            numbers_list = None
            # Adding sorting key if not present
            if sorting_key not in self.sorted_dict.keys(): 
                self.sorted_dict[sorting_key] = [] if sorting_key == "Mixed" else {}

            if sorting_key == "Mixed":
                numbers_list = self.sorted_dict[sorting_key]
            else:
                if qty not in self.sorted_dict[sorting_key].keys():
                    self.sorted_dict[sorting_key][qty] = []
                numbers_list = self.sorted_dict[sorting_key][qty]
            numbers_list += page_nums
            
        except Exception as e:
            print(e)
            
    def create_single_pdf_file(self, pdf_name, page_numbers):
        try:
            reader = PdfReader(self.label_filepath); writer = PdfWriter()
            print(pdf_name, page_numbers)
            # adding pages to the writer
            for page in page_numbers:
                writer.add_page(reader.pages[page-1])
                
            page_count = len(page_numbers)
            order_count = int(page_count/2) if self.platform == "Amazon" else page_count
            
            sorted_pdf_file = f"{re.sub(r"[\|\.\/]*",r"",pdf_name)} - {order_count} order{"s" if order_count > 1 else ""}.pdf"
        except Exception as e:
            print(e)
        else:
            if writer:
                if sorted_pdf_file:
                    out_filepath = os.path.join(self.output_folder, sorted_pdf_file)
                    with open(out_filepath, "wb") as out_pdf:
                        writer.write(out_pdf)        
            
    def create_sorted_pdf_files(self):
        summary_dict = self.create_sorted_summary()
        
        #pprint(summary_dict.keys())
        
        if len(summary_dict.keys()) == 0:
            sys.exit("Cannot sort with empty summary...")
            
        order_count = None; page_numbers = None
        output_file = None 
        
        # Create output folder if not created already.
        if not os.path.exists(self.output_folder):
            os.makedirs(self.output_folder)
            print(f"Created output folder : {self.output_folder}")
        
        # save the summary as a json file to the output folder
        with open(f"{self.output_folder}/summary.json","w") as summary_json:
            json.dump(summary_dict, summary_json)
            
        try:
            print(f"Sorted Summary :")
            for sorting_key, value in summary_dict.items():
                # Assigning output file name and its pages according to order type
                # Mixed orders
                if type(value) == list:
                    self.create_single_pdf_file(pdf_name=sorting_key, page_numbers=value)
                # single item orders
                elif type(value) == dict:
                    #print(f"Writing Single item order",end=", ")
                    for qty,page_list in value.items():
                        #print(f"Detected more than one qty.")
                        self.create_single_pdf_file(pdf_name=f"{sorting_key} - {qty}", page_numbers=page_list)
        except Exception as e:
            print(f"Err : {e}")



from setuptools import setup, find_packages
from pathlib import Path


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


setup(
    name="label_sorter",
    version="0.3",
    packages = find_packages(),
    install_requires = ['asarPy==1.0.1', 'blinker==1.9.0', 'build==1.2.2.post1', 'certifi==2025.7.14', 'cffi==1.17.1', 'charset-normalizer==3.4.2', 'click==8.2.1', 'cryptography==45.0.5', 'docutils==0.21.2', 'et_xmlfile==2.0.0', 'greenlet==3.2.3', 'id==1.5.0', 'idna==3.10', 'iniconfig==2.1.0', 'itsdangerous==2.2.0', 'jaraco.classes==3.4.0', 'jaraco.context==6.0.1', 'jaraco.functools==4.2.1', 'jeepney==0.9.0', 'keyring==25.6.0', 'markdown-it-py==3.0.0', 'MarkupSafe==3.0.2', 'mdurl==0.1.2', 'more-itertools==10.7.0', 'nh3==0.3.0', 'packaging==25.0', 'pdfminer.six==20250506', 'pdfplumber==0.11.7', 'pillow==11.3.0', 'pluggy==1.6.0', 'pycparser==2.22', 'Pygments==2.19.2', 'pypdf==5.8.0', 'PyPDF2==3.0.1', 'pypdfium2==4.30.1', 'pyproject_hooks==1.2.0', 'pytest==8.4.1', 'python-dateutil==2.9.0.post0', 'pytz==2025.2', 'readme_renderer==44.0', 'requests==2.32.4', 'requests-toolbelt==1.0.0', 'rfc3986==2.0.0', 'rich==14.1.0', 'SecretStorage==3.3.3', 'setuptools==80.9.0', 'six==1.17.0', 'termcolor==3.1.0', 'twine==6.1.0', 'typing_extensions==4.14.1', 'tzdata==2025.2', 'urllib3==2.5.0', 'Werkzeug==3.1.3', 'wheel==0.45.1'],
    author = "Harry19967",
    author_email="harilalsunil2@gmail.com",
    description = "Library to sort Amazon and Shopify shipping labels",
    url= "https://github.com/harilal766/Ecommerce-label-sorter",
    license = "MIT",
    classifiers = [
    "Programming Language :: Python :: 3",
    "Operating System :: OS Independent"
    ],
    python_requires = ">=3.9",
    long_description=long_description,
    long_description_content_type='text/markdown'
)


